
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/

#include "StdAfx.h"
#include "AddressItem.h"

using namespace AutomateIT::CalcItems;

class NotSimbol
{
public:
    bool operator()(wchar_t character)
    {
        return (character < 'A' || character > 'Z');
    }
};

class NotDigit
{
public:
    bool operator()(wchar_t character)
    {
        return (character < '0' || character > '9');
    }
};

class SymbolAccumulator
{
public:
    SymbolAccumulator() : result_(0) 
    {

    }
    void operator ()(wchar_t character)
    {
        result_ = (L'Z' - L'A' + 1) * result_ + (character - L'A' + 1);
    }

    long GetResult() const
    {
        return result_;
    }

private:

    long result_;

};

CAddressItem::CAddressItem(const std::wstring& address)
{
    std::wstring right, left;
    SplitComponents(address, right, left);
    address_.x = TanslateFromSymbols(right);
    address_.y = TanslateFromDigits(left);
}

CAddressItem::CAddressItem( const std::wstring& address, Position position )
{
    std::wstring right, left;
    SplitComponents(address, right, left);
    if (position == First)
    {
        if (!right.empty())
        {
            address_.x = TanslateFromSymbols(right);
            address_.y = MinRow();
        }
        else
        {
            address_.x = MinCol();
            address_.y = TanslateFromDigits(left);
        }
    }
    if (position == Second)
    {
        if (!right.empty())
        {
            address_.x = TanslateFromSymbols(right);
            address_.y = MaxRow();
        }
        else
        {
            address_.x = MaxCol();
            address_.y = TanslateFromDigits(left);
        }
    }
}

CAddressItem::CAddressItem( const CPoint& point ) : 
    address_(point.x - 1, point.y - 1)
{

}

CAddressItem::~CAddressItem()
{
}

const CPoint& CAddressItem::GetAddress() const
{
    return address_;
}


long CAddressItem::TanslateFromSymbols( const std::wstring& address ) const
{
    if (address.empty())
    {
        throw std::exception("Empty symbol part");
    }
    std::wstring::const_iterator beg = address.begin();
    if (address.at(0) == bax_)
    {
        beg++;
    }
    return for_each(beg, address.end(), SymbolAccumulator()).GetResult() - 1;
}

long CAddressItem::TanslateFromDigits( const std::wstring& address ) const
{
    if (address.empty())
    {
        throw std::exception("Empty digit part");
    }
    std::wstring::const_iterator beg = address.begin();
    if (address.at(0) == bax_)
    {
        beg++;
    }
    std::wstringstream stream(&(*beg));
    long result = 0;
    stream >> result;
    return result - 1;
}



bool CAddressItem::HasBothComponents( const std::wstring& address )
{
    std::wstring right, left;
    SplitComponents(address, right, left);
    return !(right.empty() || left.empty());
}

void CAddressItem::SplitComponents( const std::wstring& address, std::wstring& right, std::wstring& left )
{
    std::wstring::const_iterator beg = address.begin();
    if (bax_ == address.at(0))
    {
        beg++;
    }
    std::wstring::const_iterator sep = std::find_if(beg, address.end(), NotSimbol());
    std::wstring::const_iterator firstEnd = sep;
    if (firstEnd != address.end() && bax_ == *firstEnd)
    {
        sep++;
    }
    std::wstring::const_iterator iter = std::find_if(sep, address.end(), NotDigit());
    right.assign(beg, firstEnd);
    left.assign(sep, iter);
}

std::wstring CAddressItem::ToString() const
{
    return TanslateToSymbols(address_.x) + TanslateToDigits(address_.y);
}

std::wstring CAddressItem::TanslateToSymbols( long num ) const
{
    std::wstring result;
    long rest = num;
    const int symCount = L'Z' - L'A' + 1;
    do
    {
        wchar_t symb = L'A' + wchar_t(rest % symCount);
        result = std::wstring(&symb, 1) + result;
        rest = rest / symCount;
        rest -= 1;
    }
    while (rest >= 0);
    return result;
}

std::wstring CAddressItem::TanslateToDigits( long num ) const
{
    std::wstringstream stream;
    stream << num + 1;
    return stream.str();
}

CAddressItem& CAddressItem::operator+=( const CPoint& point )
{
    address_ += point;
    return *this;
}
