
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/

#ifndef _ADDRESS_ITEM_H_
#define _ADDRESS_ITEM_H_

#pragma once

#include <string>
#include <memory>
#include <atlbase.h>
#include <atltypes.h>
#include <atlcom.h>

namespace AutomateIT
{
    namespace CalcItems
    {

        class CAddressItem
        {
        public:

            enum Position
            {
                First,
                Second
            };

            CAddressItem(const std::wstring& address);
            CAddressItem(const std::wstring& address, Position position);
            CAddressItem(const CPoint& point);
            virtual ~CAddressItem();

            const CPoint& GetAddress() const;
            std::wstring ToString() const;

            CAddressItem& operator += (const CPoint& point);

        public:

            static bool HasBothComponents(const std::wstring& address);
            
            inline static wchar_t GetBax() { return bax_; }
            
            inline static long MinCol() { return minCol_; }
            inline static long MaxCol() { return maxCol_; }
            inline static long MinRow() { return minRow_; }
            inline static long MaxRow() { return maxRow_; }

        private:

            long TanslateFromSymbols(const std::wstring& address) const ;
            long TanslateFromDigits(const std::wstring& address) const ;
            std::wstring TanslateToSymbols(long num) const;
            std::wstring TanslateToDigits(long num) const;
            static void SplitComponents(const std::wstring& address, std::wstring& right, std::wstring& left);

            CPoint address_;

        private:

            static const long minCol_ = 0x00000000;
            static const long maxCol_ = 0x000000ff;
            static const long minRow_ = 0x00000000;
            static const long maxRow_ = 0x0000ffff;

            static const wchar_t bax_ = L'$';
        };

        typedef std::auto_ptr<CAddressItem> CAddressItemPtr;

    }
}
#endif // _ADDRESS_ITEM_H_
