
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/

#include "StdAfx.h"
#include "Worksheet.h"

using namespace AutomateIT;
using namespace AutomateIT::Utils;
using namespace AutomateIT::CalcItems;
using namespace AutomateIT::ExcelClone;
using namespace AutomateIT::OpenOfficeWrapper;

CWorksheet::CWorksheet():
m_pParentWorkbook(NULL),
m_ParentApplication(NULL), 
m_allCells(NULL)
{
    TRACINGMETHOD;
}

CWorksheet::~CWorksheet()
{
    TRACINGMETHOD;
}



HRESULT CWorksheet::Initialize(CApplication* pParentApplication, CWorkbook *pParentWorkbook, COORangePtr pSheet)
{
    TRACINGMETHOD;
    m_ParentApplication = pParentApplication;
    m_pParentWorkbook = pParentWorkbook;
    m_pSheet = pSheet;
    CRangeObject* pRange(NULL);
    hr = CRangeObject::CreateInstance(&pRange);
    CHECKERROR;
    m_allCells = pRange;
    hr = m_allCells->Initialize(m_ParentApplication, this, m_pSheet);
    CHECKERROR;
    return hr;
}



CWorkbook* CWorksheet::GetParentWorkbook()
{
    return m_pParentWorkbook;
}




HRESULT STDMETHODCALLTYPE CWorksheet::get_Application( _Application **RHS )

{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
} 

HRESULT STDMETHODCALLTYPE CWorksheet::get_Creator( XlCreator *RHS )

{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
} 

HRESULT STDMETHODCALLTYPE CWorksheet::get_Parent( IDispatch **RHS )

{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
} 

HRESULT STDMETHODCALLTYPE CWorksheet::Activate( long lcid )

{
    TRACINGMETHOD;
    hr = m_pParentWorkbook->GetOODocument()->put_ActiveSheet(m_pSheet);
    CHECKERROR;
    return hr;
} 

HRESULT STDMETHODCALLTYPE CWorksheet::Copy( VARIANT Before,    VARIANT After,    long lcid )

{
    TRACINGMETHOD;

    enum eType { tBefore, tAfter, tJust } type = tJust;

    IDispatchPtr pdWhere(NULL);
    if (Before.vt == VT_DISPATCH)
    {
        pdWhere = Before.pdispVal;
        type = tBefore;
    }
    if (After.vt == VT_DISPATCH)
    {
        pdWhere = After.pdispVal;
        type = tAfter;
    }
    CComBSTR name;

    CComPtr<Sheets> pSheets(NULL);

    if (type != tJust)
    {
        CComPtr<_Worksheet> pWorksheet(NULL);
        hr = pdWhere.QueryInterface(&pWorksheet);
        CHECKERROR;

        hr = pWorksheet->get_Name(&name);
        CHECKERROR;

        CWorksheet* pWorksheetImpl = reinterpret_cast<CWorksheet*>(pWorksheet.p);
        hr = pWorksheetImpl->GetParentWorkbook()->get_Sheets(&pSheets);
        CHECKERROR;
    }
    else
    {
        hr = m_pParentWorkbook->get_Sheets(&pSheets);
        CHECKERROR;
    }
    

    IDispatchPtr pdNewSheet(NULL);

    if (type == tBefore)
    {
        hr = pSheets->Add(CComVariant(name), CComVariant(), CComVariant(), CComVariant(xlWorksheet), lcid, &pdNewSheet);
    }
    else if (type == tAfter)
    {
        hr = pSheets->Add(CComVariant(), CComVariant(name), CComVariant(), CComVariant(xlWorksheet), lcid, &pdNewSheet);
    }
    else
    {
        hr = pSheets->Add(CComVariant(), CComVariant(), CComVariant(), CComVariant(xlWorksheet), lcid, &pdNewSheet);
    }
    CHECKERROR;

    CComPtr<_Worksheet> pNewSheet(NULL);
    hr = pdNewSheet->QueryInterface(&pNewSheet);
    CHECKERROR;

    CComPtr<Range> pSourcePange(NULL);
    hr = get_Columns(&pSourcePange);
    CHECKERROR;
    
    CComPtr<Range> pDestPange(NULL);
    hr = pNewSheet->get_Columns(&pDestPange);
    CHECKERROR;
    
    CComVariant res;
    hr = pSourcePange->Copy(CComVariant(pDestPange.p), &res);
    CHECKERROR;

    CComBSTR srcName;
    hr = get_Name(&srcName);
    CHECKERROR;

    srcName.Append(OLESTR(" 2"));
    
    hr = pNewSheet->put_Name(srcName);
    CHECKERROR;

    return hr;
} 

HRESULT STDMETHODCALLTYPE CWorksheet::Delete( long lcid )

{
    TRACINGMETHOD;
    CComBSTR name;
    hr = get_Name(&name);
    CHECKERROR;
    
    CComPtr<Sheets> pSheets(NULL);
    hr = m_pParentWorkbook->get_Sheets(&pSheets);
    CHECKERROR;

    CSheets* pSheetsImpl = reinterpret_cast<CSheets*>(pSheets.p);
    hr = pSheetsImpl->GetOOSheetObject()->RemoveByName(name);
    CHECKERROR;
    
    return hr;
} 

HRESULT STDMETHODCALLTYPE CWorksheet::get_CodeName( BSTR *RHS )

{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
} 

HRESULT STDMETHODCALLTYPE CWorksheet::get__CodeName( BSTR *RHS )

{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
} 

HRESULT STDMETHODCALLTYPE CWorksheet::put__CodeName( BSTR RHS )

{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
} 

HRESULT STDMETHODCALLTYPE CWorksheet::get_Index( long lcid,    long *RHS )

{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
} 

HRESULT STDMETHODCALLTYPE CWorksheet::Move( VARIANT Before,    VARIANT After,    long lcid )

{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
} 

HRESULT STDMETHODCALLTYPE CWorksheet::get_Name( BSTR *pbstrName )
{ 
    TRACINGMETHOD; 
    hr = m_pSheet->GetName(pbstrName);
    CHECKERROR;
    return hr;
} 

HRESULT STDMETHODCALLTYPE CWorksheet::put_Name( BSTR bstrName )
{ 
    TRACINGMETHOD; 
    hr = m_pSheet->SetName(bstrName);
    CHECKERROR;
    return hr;
} 

HRESULT STDMETHODCALLTYPE CWorksheet::get_Next( IDispatch **RHS )

{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
} 

HRESULT STDMETHODCALLTYPE CWorksheet::get_OnDoubleClick( long lcid,    BSTR *RHS )

{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
} 

HRESULT STDMETHODCALLTYPE CWorksheet::put_OnDoubleClick( long lcid,    BSTR RHS )

{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
} 

HRESULT STDMETHODCALLTYPE CWorksheet::get_OnSheetActivate( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
} 

HRESULT STDMETHODCALLTYPE CWorksheet::put_OnSheetActivate( 
    long lcid,
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_OnSheetDeactivate( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::put_OnSheetDeactivate( 
    long lcid,
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_PageSetup( 
    PageSetup **RHS)
{
    TRACINGMETHOD;
    CPageSetupObject* pPageSetupObject(NULL);
    hr = CPageSetupObject::CreateInstance(&pPageSetupObject);
    CHECKERROR;
    CPageSetupPtr pPageSetup(pPageSetupObject);

    COOStyleFamiliesPtr pStyleFamilies(NULL);
    hr = m_pParentWorkbook->GetOODocument()->GetDocument()->GetStyleFamilies(&pStyleFamilies);
    CHECKERROR;


    COOPageStylesPtr pPageStyles(NULL);
    hr = pStyleFamilies->GetPageStyles(&pPageStyles);
    CHECKERROR;

    CComBSTR styleName(NULL);
    hr = m_pSheet->get_PageStyle(&styleName);
    CHECKERROR;

    COOPageSetupStylePtr pStyle(NULL);
    hr = pPageStyles->GetByName(styleName, &pStyle);
    CHECKERROR;

    hr = pPageSetup->Initialize(m_allCells.p, pStyle);
    CHECKERROR;

    (*RHS) = pPageSetup;
    (*RHS)->AddRef();
    
    return hr;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_Previous( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::_PrintOut( 
    VARIANT From,
    VARIANT To,
    VARIANT Copies,
    VARIANT Preview,
    VARIANT ActivePrinter,
    VARIANT PrintToFile,
    VARIANT Collate,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::PrintPreview( 
    VARIANT EnableChanges,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::_Protect( 
    VARIANT Password,
    VARIANT DrawingObjects,
    VARIANT Contents,
    VARIANT Scenarios,
    VARIANT UserInterfaceOnly,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_ProtectContents( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_ProtectDrawingObjects( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_ProtectionMode( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_ProtectScenarios( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::_SaveAs( 
    BSTR Filename,
    VARIANT FileFormat,
    VARIANT Password,
    VARIANT WriteResPassword,
    VARIANT ReadOnlyRecommended,
    VARIANT CreateBackup,
    VARIANT AddToMru,
    VARIANT TextCodepage,
    VARIANT TextVisualLayout,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::Select( 
    VARIANT Replace,
    long lcid)
{
    TRACINGMETHOD;
    return Activate(lcid);
}

HRESULT STDMETHODCALLTYPE CWorksheet::Unprotect( 
    VARIANT Password,
    long lcid)
{
    TRACINGMETHOD;
    CComVariant pass(Password);
    CComBSTR passStr;
    if (pass.vt == VT_EMPTY || pass.vt == VT_NULL)
    {
        passStr = OLESTR("");
    }
    else
    {
        hr = pass.ChangeType(VT_BSTR);
        CHECKERROR;
        passStr = pass.bstrVal;
    }
    hr = m_pSheet->Unprotect(passStr);
    CHECKERROR;
    return hr;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_Visible( 
    long lcid,
    XlSheetVisibility *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::put_Visible( 
    long lcid,
    XlSheetVisibility RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_Shapes( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_TransitionExpEval( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::put_TransitionExpEval( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::Arcs( 
    VARIANT Index,
    long lcid,
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_AutoFilterMode( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::put_AutoFilterMode( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::SetBackgroundPicture( 
    BSTR Filename)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::Buttons( 
    VARIANT Index,
    long lcid,
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::Calculate( 
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_EnableCalculation( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::put_EnableCalculation( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_Cells( 
    Range **ppRange)
{ 
    TRACINGMETHOD; 
    if (ppRange == NULL)
    {
        return E_POINTER;
    }
    *ppRange = m_allCells;
    (*ppRange)->AddRef();
    return hr;
}

HRESULT STDMETHODCALLTYPE CWorksheet::ChartObjects( 
    VARIANT Index,
    long lcid,
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::CheckBoxes( 
    VARIANT Index,
    long lcid,
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::CheckSpelling( 
    VARIANT CustomDictionary,
    VARIANT IgnoreUppercase,
    VARIANT AlwaysSuggest,
    VARIANT SpellLang,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_CircularReference( 
    long lcid,
    Range **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::ClearArrows( 
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_Columns( 
    Range **RHS)
{ 
    TRACINGMETHOD;
    CRange* range(NULL);
    hr = get_Cells(reinterpret_cast<Range**>(&range));
    CHECKERROR;

    *RHS = range;
    range->SetInternalType(xlColumns);

    return hr;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_ConsolidationFunction( 
    long lcid,
    XlConsolidationFunction *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_ConsolidationOptions( 
    long lcid,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_ConsolidationSources( 
    long lcid,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_DisplayAutomaticPageBreaks( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::put_DisplayAutomaticPageBreaks( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::Drawings( 
    VARIANT Index,
    long lcid,
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::DrawingObjects( 
    VARIANT Index,
    long lcid,
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::DropDowns( 
    VARIANT Index,
    long lcid,
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_EnableAutoFilter( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::put_EnableAutoFilter( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_EnableSelection( 
    XlEnableSelection *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::put_EnableSelection( 
    XlEnableSelection RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_EnableOutlining( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::put_EnableOutlining( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_EnablePivotTable( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::put_EnablePivotTable( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::Evaluate( 
    VARIANT Name,
    long lcid,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::_Evaluate( 
    VARIANT Name,
    long lcid,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_FilterMode( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::ResetAllPageBreaks( void)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::GroupBoxes( 
    VARIANT Index,
    long lcid,
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::GroupObjects( 
    VARIANT Index,
    long lcid,
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::Labels( 
    VARIANT Index,
    long lcid,
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::Lines( 
    VARIANT Index,
    long lcid,
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::ListBoxes( 
    VARIANT Index,
    long lcid,
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_Names( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::OLEObjects( 
    VARIANT Index,
    long lcid,
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_OnCalculate( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::put_OnCalculate( 
    long lcid,
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_OnData( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::put_OnData( 
    long lcid,
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_OnEntry( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::put_OnEntry( 
    long lcid,
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::OptionButtons( 
    VARIANT Index,
    long lcid,
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_Outline( 
    IDispatch **RHS)
{ 
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::Ovals( 
    VARIANT Index,
    long lcid,
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::Paste( 
    VARIANT Destination,
    VARIANT Link,
    long lcid)
{ 
    TRACINGMETHOD; 

    COODispatchHelperPtr pDispatchHelper(NULL);
    hr = m_ParentApplication->GetOOApplication()->CreateDispatchHelper(&pDispatchHelper);

    IDispatchPtr pFrame(NULL);
    hr = GetParentWorkbook()->GetOODocument()->GetDocument()->GetFrame(&pFrame);
    CHECKERROR;

    hr = pDispatchHelper->ExecuteDispatch(pFrame, OLESTR(".uno:Paste"));
    CHECKERROR;
    return hr;
}

HRESULT STDMETHODCALLTYPE CWorksheet::_PasteSpecial( 
    VARIANT Format,
    VARIANT Link,
    VARIANT DisplayAsIcon,
    VARIANT IconFileName,
    VARIANT IconIndex,
    VARIANT IconLabel,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::Pictures( 
    VARIANT Index,
    long lcid,
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::PivotTables( 
    VARIANT Index,
    long lcid,
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::PivotTableWizard( 
    VARIANT SourceType,
    VARIANT SourceData,
    VARIANT TableDestination,
    VARIANT TableName,
    VARIANT RowGrand,
    VARIANT ColumnGrand,
    VARIANT SaveData,
    VARIANT HasAutoFormat,
    VARIANT AutoPage,
    VARIANT Reserved,
    VARIANT BackgroundQuery,
    VARIANT OptimizeCache,
    VARIANT PageFieldOrder,
    VARIANT PageFieldWrapCount,
    VARIANT ReadData,
    VARIANT Connection,
    long lcid,
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_Range( 
    VARIANT Cell1,
    VARIANT Cell2,
    Range **ppRange)
{ 
    TRACINGMETHOD; 
    return m_allCells->get_Range(Cell1, Cell2, ppRange);
}

HRESULT STDMETHODCALLTYPE CWorksheet::Rectangles( 
    VARIANT Index,
    long lcid,
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_Rows( 
    Range **RHS)
{
    TRACINGMETHOD;
    CRange* range(NULL);
    hr = get_Cells(reinterpret_cast<Range**>(&range));
    CHECKERROR;

    *RHS = range;
    range->SetInternalType(xlRows);

    return hr;
}

HRESULT STDMETHODCALLTYPE CWorksheet::Scenarios( 
    VARIANT Index,
    long lcid,
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_ScrollArea( 
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::put_ScrollArea( 
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::ScrollBars( 
    VARIANT Index,
    long lcid,
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::ShowAllData( 
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::ShowDataForm( 
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::Spinners( 
    VARIANT Index,
    long lcid,
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_StandardHeight( 
    long lcid,
    double *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_StandardWidth( 
    long lcid,
    double *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::put_StandardWidth( 
    long lcid,
    double RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::TextBoxes( 
    VARIANT Index,
    long lcid,
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_TransitionFormEntry( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::put_TransitionFormEntry( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_Type( 
    long lcid,
    XlSheetType *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_UsedRange( 
    long lcid,
    Range **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_HPageBreaks( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_VPageBreaks( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_QueryTables( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_DisplayPageBreaks( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::put_DisplayPageBreaks( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_Comments( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_Hyperlinks( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::ClearCircles( void)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::CircleInvalid( void)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get__DisplayRightToLeft( 
    long lcid,
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::put__DisplayRightToLeft( 
    long lcid,
    long RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_AutoFilter( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_DisplayRightToLeft( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::put_DisplayRightToLeft( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_Scripts( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::PrintOut( 
    VARIANT From,
    VARIANT To,
    VARIANT Copies,
    VARIANT Preview,
    VARIANT ActivePrinter,
    VARIANT PrintToFile,
    VARIANT Collate,
    VARIANT PrToFileName,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::_CheckSpelling( 
    VARIANT CustomDictionary,
    VARIANT IgnoreUppercase,
    VARIANT AlwaysSuggest,
    VARIANT SpellLang,
    VARIANT IgnoreFinalYaa,
    VARIANT SpellScript,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_Tab( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_MailEnvelope( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::SaveAs( 
    BSTR Filename,
    VARIANT FileFormat,
    VARIANT Password,
    VARIANT WriteResPassword,
    VARIANT ReadOnlyRecommended,
    VARIANT CreateBackup,
    VARIANT AddToMru,
    VARIANT TextCodepage,
    VARIANT TextVisualLayout,
    VARIANT Local)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_CustomProperties( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_SmartTags( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_Protection( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::PasteSpecial( 
    VARIANT Format,
    VARIANT Link,
    VARIANT DisplayAsIcon,
    VARIANT IconFileName,
    VARIANT IconIndex,
    VARIANT IconLabel,
    VARIANT NoHTMLFormatting,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::Protect( 
    VARIANT Password,
    VARIANT DrawingObjects,
    VARIANT Contents,
    VARIANT Scenarios,
    VARIANT UserInterfaceOnly,
    VARIANT AllowFormattingCells,
    VARIANT AllowFormattingColumns,
    VARIANT AllowFormattingRows,
    VARIANT AllowInsertingColumns,
    VARIANT AllowInsertingRows,
    VARIANT AllowInsertingHyperlinks,
    VARIANT AllowDeletingColumns,
    VARIANT AllowDeletingRows,
    VARIANT AllowSorting,
    VARIANT AllowFiltering,
    VARIANT AllowUsingPivotTables)
{
    TRACINGMETHOD;
    CComVariant pass(Password);
    CComBSTR passStr;
    if (pass.vt == VT_EMPTY || pass.vt == VT_NULL)
    {
        passStr = OLESTR("");
    }
    else
    {
        hr = pass.ChangeType(VT_BSTR);
        CHECKERROR;
        passStr = pass.bstrVal;
    }
    hr = m_pSheet->Protect(passStr);
    CHECKERROR;
    return hr;
}

HRESULT STDMETHODCALLTYPE CWorksheet::get_ListObjects( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::XmlDataQuery( 
    BSTR XPath,
    VARIANT SelectionNamespaces,
    VARIANT Map,
    Range **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CWorksheet::XmlMapQuery( 
    BSTR XPath,
    VARIANT SelectionNamespaces,
    VARIANT Map,
    Range **RHS) { 
        TRACINGMETHOD; 
        TRACINGNOTIMPL;
        return S_OK; 
}


