/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/

#include "StdAfx.h"
#include "OOCalcDocument.h"

using namespace AutomateIT::Utils;
using namespace AutomateIT::OpenOfficeWrapper;

COOCalcDocument::COOCalcDocument()
{
}

COOCalcDocument::~COOCalcDocument()
{
}

HRESULT COOCalcDocument::Attach( COODocumentPtr pDoc )
{
    m_pDoc = pDoc;
    return S_OK;
}

HRESULT COOCalcDocument::Detach()
{
    m_pDoc = 0;
    return S_OK;
}

COODocumentPtr COOCalcDocument::GetDocument() const
{
    return m_pDoc;
}

HRESULT COOCalcDocument::GetSheets( COOSheets** ppSheets )
{

    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_METHOD, &result, m_pDoc->GetIDispatchInstance(), L"getSheets", 0);
    CHECKERROR;
    IDispatchPtr dispatch = result.pdispVal;
    
    COOSheetsObject* pSheets(NULL); 
    hr = COOSheetsObject::CreateInstance(&pSheets);
    CHECKERROR;

    COOSheetsPtr pSheetsPtr(pSheets); 
    hr = pSheetsPtr->Initialize(dispatch);
    CHECKERROR;

    (*ppSheets) = pSheetsPtr;
    (*ppSheets)->AddRef();

    return hr;
}

HRESULT COOCalcDocument::get_NumberFormats( COONumberFormats** ppOONumberFormats )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pDoc->GetIDispatchInstance(), L"NumberFormats", 0);
    CHECKERROR;
    IDispatchPtr dispatch = result.pdispVal;

    COONumberFormatsObject* pNumberFormats(NULL); 
    hr = COONumberFormatsObject::CreateInstance(&pNumberFormats);
    CHECKERROR;

    COONumberFormatsPtr pNumberFormatsPtr(pNumberFormats); 
    hr = pNumberFormatsPtr->Initialize(dispatch);
    CHECKERROR;

    (*ppOONumberFormats) = pNumberFormatsPtr;
    (*ppOONumberFormats)->AddRef();

    return hr;
}


HRESULT COOCalcDocument::get_CurrentSelection( COORange** pOORange )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pDoc->GetIDispatchInstance(), L"CurrentSelection", 0);
    CHECKERROR;

    IDispatchPtr dispatch = result.pdispVal;

    COORangeObject* pOORangeObject(NULL); 
    hr = COORangeObject::CreateInstance(&pOORangeObject);
    CHECKERROR;

    COORangePtr pRange(pOORangeObject); 
    hr = pRange->Initialize(dispatch);
    CHECKERROR;

    (*pOORange) = pRange;
    (*pOORange)->AddRef();

    return hr;
}

HRESULT COOCalcDocument::put_ActiveSheet( COORange* pOORange )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pDoc->GetIDispatchInstance(), L"CurrentController", 0);
    CHECKERROR;
    IDispatchPtr pController = result.pdispVal;

    result.Clear();
    hr = AutoWrapper::Execute(DISPATCH_PROPERTYPUT, &result, pController, L"ActiveSheet", 1, CComVariant(pOORange->m_pdRange.p));
    CHECKERROR;
    return hr;
}

HRESULT COOCalcDocument::get_ActiveSheet( COORange** ppOORange )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pDoc->GetIDispatchInstance(), L"CurrentController", 0);
    CHECKERROR;
    IDispatchPtr pController = result.pdispVal;

    result.Clear();
    hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, pController, L"ActiveSheet", 0);
    CHECKERROR;
    IDispatchPtr dispatch = result.pdispVal;

    COORangeObject* pOORangeObject(NULL); 
    hr = COORangeObject::CreateInstance(&pOORangeObject);
    CHECKERROR;

    COORangePtr pRange(pOORangeObject); 
    hr = pRange->Initialize(dispatch);
    CHECKERROR;

    (*ppOORange) = pRange;
    (*ppOORange)->AddRef();

    return hr;
}

HRESULT COOCalcDocument::Select( COORange* pOORange )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pDoc->GetIDispatchInstance(), L"CurrentController", 0);
    CHECKERROR;
    IDispatchPtr pController = result.pdispVal;

    result.Clear();
    hr = AutoWrapper::Execute(DISPATCH_METHOD, &result, pController, L"select", 1, CComVariant(pOORange->m_pdRange.p));
    CHECKERROR;
    return hr;
}

HRESULT COOCalcDocument::Protect(BSTR bstrPassword)
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_METHOD, &result, m_pDoc->GetIDispatchInstance(), L"protect", 1, CComVariant(bstrPassword));
    CHECKERROR;
    return hr;
}

HRESULT COOCalcDocument::Unprotect(BSTR bstrPassword)
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_METHOD, &result, m_pDoc->GetIDispatchInstance(), L"unprotect", 1, CComVariant(bstrPassword));
    CHECKERROR;
    return hr;
}
