
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/
#include "stdafx.h"
#include "OODocument.h"

using namespace AutomateIT::Utils;
using namespace AutomateIT::OpenOfficeWrapper;

COODocument::COODocument() : m_pApp(NULL), m_pInstance(NULL)
{
}

HRESULT COODocument::Initialize(COOApplication* pParentApplication, IDispatchPtr pdDoc)
{
    m_pInstance = pdDoc;
    m_pApp = pParentApplication;
    return S_OK;
}

COOApplication* COODocument::GetApplication() const
{
    return m_pApp;
}

HRESULT COODocument::SaveFile(CComVariant vFileName, CComVariant vFormatName)
{
    CComVariant vRes;
	IDispatch* pdProperty = NULL;
	CComVariant vParam;
	
    SAFEARRAY FAR* pPropVals;

    if (vFormatName.vt != VT_ERROR)
    {
        HRESULT hres = m_pApp->GetDispatchPropertyValue(&pdProperty); 
        if (hres != S_OK)
        {
            return hres;
        }
		
	    // Set PropertyValue data
        hres = m_pApp->SetPropertyValue(pdProperty, L"FilterName", vFormatName);
        if (hres != S_OK)
        {
            pdProperty->Release();
            return hres;
        }

	    // Init params
		pPropVals = SafeArrayCreateVector( VT_DISPATCH, 0, 1 );
	    long ix = 0; 
	    hres = SafeArrayPutElement( pPropVals, &ix, pdProperty); 
	    
		vParam.vt = VT_DISPATCH | VT_ARRAY;
	    vParam.parray = pPropVals;

    } else 
    {
        vParam.vt = VT_ARRAY | VT_VARIANT;
        vParam.parray = NULL;
    }

	// Call StoreToURL for save document to file
	
	HRESULT hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, m_pInstance, L"storeToURL", 2, vFileName, vParam);
    
	if (pdProperty != NULL)
    {
        pdProperty->Release();
    }
    return hres;
}

HRESULT COODocument::Save()
{
    // Call store for save document to file
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_METHOD, &result, m_pInstance, L"store", 0);
    CHECKERROR;
    return hr;
}

HRESULT COODocument::Close()
{
    if (m_pInstance == NULL)
    {
        return E_INVALIDARG;
    }
    CComVariant vRes;
    return AutoWrapper::Execute(DISPATCH_METHOD, &vRes, m_pInstance, L"dispose", 0);
}

COODocument::~COODocument()
{
}


HRESULT COODocument::GetStyleFamilies( COOStyleFamilies** ppStyleFamilies )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_METHOD, &result, m_pInstance, L"getStyleFamilies", 0);
    CHECKERROR;
    IDispatchPtr pdStyleFamilies = result.pdispVal;
    COOStyleFamiliesObject* pStyleFamiliesObject(NULL);
    hr = COOStyleFamiliesObject::CreateInstance(&pStyleFamiliesObject);
    CHECKERROR;
    COOStyleFamiliesPtr pStyleFamilies(pStyleFamiliesObject);
    hr = pStyleFamilies->Initialize(pdStyleFamilies);
    CHECKERROR;
    (*ppStyleFamilies) = pStyleFamilies;
    (*ppStyleFamilies)->AddRef();
    return hr;

}


HRESULT COODocument::Activate()
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pInstance, L"CurrentController", 0);
    CHECKERROR;
    IDispatchPtr pController = result.pdispVal;
    
    result.Clear();
    hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, pController, L"Frame", 0);
    CHECKERROR;
    IDispatchPtr pFrame = result.pdispVal;;

    result.Clear();
    hr = AutoWrapper::Execute(DISPATCH_METHOD, &result, pFrame, L"Activate", 0);
    CHECKERROR;

    return hr;
}

HRESULT COODocument::SetVisible( bool bVisible )
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pInstance, L"CurrentController", 0);
    CHECKERROR;
    IDispatchPtr pController = result.pdispVal;

    result.Clear();
    hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, pController, L"Frame", 0);
    CHECKERROR;
    IDispatchPtr pFrame = result.pdispVal;;

    result.Clear();
    hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, pFrame, L"ContainerWindow", 0);
    CHECKERROR;
    IDispatchPtr pWindow = result.pdispVal;;

    result.Clear();
    hr = AutoWrapper::Execute(DISPATCH_METHOD, &result, pWindow, L"setVisible", 1, CComVariant(bVisible));
    CHECKERROR;

    return hr;
}

HRESULT COODocument::GetFrame(IDispatch** pdFrame)
{
    CComVariant result;
    HRESULT hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, m_pInstance, L"CurrentController", 0);
    CHECKERROR;
    IDispatchPtr pController = result.pdispVal;

    result.Clear();
    hr = AutoWrapper::Execute(DISPATCH_PROPERTYGET, &result, pController, L"Frame", 0);
    CHECKERROR;
    IDispatchPtr pFrame = result.pdispVal;

    (*pdFrame) = pFrame;
    (*pdFrame)->AddRef();

    return hr;

}


IDispatchPtr COODocument::GetIDispatchInstance() const
{
    return m_pInstance;
}


