
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/

#ifndef _WORD_RANGE_H_
#define _WORD_RANGE_H_

#include <atlcom.h>
#include "resource.h"       
#include "WordClone.h"
#include "WordParagraphFormat.h"
#include "WordStyle.h"

namespace AutomateIT
{
	namespace WordClone
	{
		class CWordDocument;
		class CWordParagraph;

		class ATL_NO_VTABLE CWordRange : 
			public CComObjectRootEx<CComSingleThreadModel>,
			public IDispatchImpl<Range, &__uuidof(Range), &LIBID_WordCloneLib, 8, 3>
		{
		public:

			CWordRange();
			virtual ~CWordRange();

			BEGIN_COM_MAP(CWordRange)
				COM_INTERFACE_ENTRY(Range)
			END_COM_MAP()

			DECLARE_PROTECT_FINAL_CONSTRUCT()

			HRESULT FinalConstruct()
			{
				return S_OK;
			}

			void FinalRelease() 
			{
			}

			// Range interface methods
		public:
			STDMETHOD(get_Text)(BSTR* prop);
			STDMETHOD(put_Text)(BSTR prop);
			STDMETHOD(get_FormattedText)(Range** prop);
			STDMETHOD(put_FormattedText)(Range* prop);
			STDMETHOD(get_Start)(long* prop);
			STDMETHOD(put_Start)(long prop);
			STDMETHOD(get_End)(long* prop);
			STDMETHOD(put_End)(long prop);
			STDMETHOD(get_Font)(_Font** prop);
			STDMETHOD(put_Font)(_Font* prop);
			STDMETHOD(get_Duplicate)(Range** prop);
			STDMETHOD(get_StoryType)(WdStoryType* prop);
			STDMETHOD(get_Tables)(IDispatch** prop);
			STDMETHOD(get_Words)(IDispatch** prop);
			STDMETHOD(get_Sentences)(IDispatch** prop);
			STDMETHOD(get_Characters)(IDispatch** prop);
			STDMETHOD(get_Footnotes)(IDispatch** prop);
			STDMETHOD(get_Endnotes)(IDispatch** prop);
			STDMETHOD(get_Comments)(IDispatch** prop);
			STDMETHOD(get_Cells)(IDispatch** prop);
			STDMETHOD(get_Sections)(Sections** prop);
			STDMETHOD(get_Paragraphs)(Paragraphs** prop);
			STDMETHOD(get_Borders)(IDispatch** prop);
			STDMETHOD(put_Borders)(IDispatch* prop);
			STDMETHOD(get_Shading)(IDispatch** prop);
			STDMETHOD(get_TextRetrievalMode)(IDispatch** prop);
			STDMETHOD(put_TextRetrievalMode)(IDispatch* prop);
			STDMETHOD(get_Fields)(IDispatch** prop);
			STDMETHOD(get_FormFields)(IDispatch** prop);
			STDMETHOD(get_Frames)(IDispatch** prop);
			STDMETHOD(get_ParagraphFormat)(_ParagraphFormat** prop);
			STDMETHOD(put_ParagraphFormat)(_ParagraphFormat* prop);
			STDMETHOD(get_ListFormat)(IDispatch** prop);
			STDMETHOD(get_Bookmarks)(IDispatch** prop);
			STDMETHOD(get_Application)(IApplication** prop);
			STDMETHOD(get_Creator)(long* prop);
			STDMETHOD(get_Parent)(LPDISPATCH* prop);
			STDMETHOD(get_Bold)(long* prop);
			STDMETHOD(put_Bold)(long prop);
			STDMETHOD(get_Italic)(long* prop);
			STDMETHOD(put_Italic)(long prop);
			STDMETHOD(get_Underline)(WdUnderline* prop);
			STDMETHOD(put_Underline)(WdUnderline prop);
			STDMETHOD(get_EmphasisMark)(WdEmphasisMark* prop);
			STDMETHOD(put_EmphasisMark)(WdEmphasisMark prop);
			STDMETHOD(get_DisableCharacterSpaceGrid)(VARIANT_BOOL* prop);
			STDMETHOD(put_DisableCharacterSpaceGrid)(VARIANT_BOOL prop);
			STDMETHOD(get_Revisions)(IDispatch** prop);
			STDMETHOD(get_Style)(VARIANT* prop);
			STDMETHOD(put_Style)(VARIANT* prop);
			STDMETHOD(get_StoryLength)(long* prop);
			STDMETHOD(get_LanguageID)(WdLanguageID* prop);
			STDMETHOD(put_LanguageID)(WdLanguageID prop);
			STDMETHOD(get_SynonymInfo)(IDispatch** prop);
			STDMETHOD(get_Hyperlinks)(IDispatch** prop);
			STDMETHOD(get_ListParagraphs)(IDispatch** prop);
			STDMETHOD(get_Subdocuments)(IDispatch** prop);
			STDMETHOD(get_GrammarChecked)(VARIANT_BOOL* prop);
			STDMETHOD(put_GrammarChecked)(VARIANT_BOOL prop);
			STDMETHOD(get_SpellingChecked)(VARIANT_BOOL* prop);
			STDMETHOD(put_SpellingChecked)(VARIANT_BOOL prop);
			STDMETHOD(get_HighlightColorIndex)(WdColorIndex* prop);
			STDMETHOD(put_HighlightColorIndex)(WdColorIndex prop);
			STDMETHOD(get_Columns)(IDispatch** prop);
			STDMETHOD(get_Rows)(IDispatch** prop);
			STDMETHOD(get_CanEdit)(long* prop);
			STDMETHOD(get_CanPaste)(long* prop);
			STDMETHOD(get_IsEndOfRowMark)(VARIANT_BOOL* prop);
			STDMETHOD(get_BookmarkID)(long* prop);
			STDMETHOD(get_PreviousBookmarkID)(long* prop);
			STDMETHOD(get_Find)(IDispatch** prop);
			STDMETHOD(get_PageSetup)(IDispatch** prop);
			STDMETHOD(put_PageSetup)(IDispatch* prop);
			STDMETHOD(get_ShapeRange)(IDispatch** prop);
			STDMETHOD(get_Case)(WdCharacterCase* prop);
			STDMETHOD(put_Case)(WdCharacterCase prop);
			STDMETHOD(get_Information)(WdInformation Type, VARIANT* prop);
			STDMETHOD(get_ReadabilityStatistics)(IDispatch** prop);
			STDMETHOD(get_GrammaticalErrors)(IDispatch** prop);
			STDMETHOD(get_SpellingErrors)(IDispatch** prop);
			STDMETHOD(get_Orientation)(WdTextOrientation* prop);
			STDMETHOD(put_Orientation)(WdTextOrientation prop);
			STDMETHOD(get_InlineShapes)(IDispatch** prop);
			STDMETHOD(get_NextStoryRange)(Range** prop);
			STDMETHOD(get_LanguageIDFarEast)(WdLanguageID* prop);
			STDMETHOD(put_LanguageIDFarEast)(WdLanguageID prop);
			STDMETHOD(get_LanguageIDOther)(WdLanguageID* prop);
			STDMETHOD(put_LanguageIDOther)(WdLanguageID prop);
			STDMETHOD(Select)();
			STDMETHOD(SetRange)(long Start, long End);
			STDMETHOD(Collapse)(VARIANT* Direction);
			STDMETHOD(InsertBefore)(BSTR Text);
			STDMETHOD(InsertAfter)(BSTR Text);
			STDMETHOD(Next)(VARIANT* Unit, VARIANT* Count, Range** prop);
			STDMETHOD(Previous)(VARIANT* Unit, VARIANT* Count, Range** prop);
			STDMETHOD(StartOf)(VARIANT* Unit, VARIANT* Extend, long* prop);
			STDMETHOD(EndOf)(VARIANT* Unit, VARIANT* Extend, long* prop);
			STDMETHOD(Move)(VARIANT* Unit, VARIANT* Count, long* prop);
			STDMETHOD(MoveStart)(VARIANT* Unit, VARIANT* Count, long* prop);
			STDMETHOD(MoveEnd)(VARIANT* Unit, VARIANT* Count, long* prop);
			STDMETHOD(MoveWhile)(VARIANT* Cset, VARIANT* Count, long* prop);
			STDMETHOD(MoveStartWhile)(VARIANT* Cset, VARIANT* Count, long* prop);
			STDMETHOD(MoveEndWhile)(VARIANT* Cset, VARIANT* Count, long* prop);
			STDMETHOD(MoveUntil)(VARIANT* Cset, VARIANT* Count, long* prop);
			STDMETHOD(MoveStartUntil)(VARIANT* Cset, VARIANT* Count, long* prop);
			STDMETHOD(MoveEndUntil)(VARIANT* Cset, VARIANT* Count, long* prop);
			STDMETHOD(Cut)();
			STDMETHOD(Copy)();
			STDMETHOD(Paste)();
			STDMETHOD(InsertBreak)(VARIANT* Type);
			STDMETHOD(InsertFile)(BSTR FileName, VARIANT* Range, VARIANT* ConfirmConversions, VARIANT* Link, VARIANT* Attachment);
			STDMETHOD(InStory)(Range* Range, VARIANT_BOOL* prop);
			STDMETHOD(InRange)(Range* Range, VARIANT_BOOL* prop);
			STDMETHOD(Delete)(VARIANT* Unit, VARIANT* Count, long* prop);
			STDMETHOD(WholeStory)();
			STDMETHOD(Expand)(VARIANT* Unit, long* prop);
			STDMETHOD(InsertParagraph)();
			STDMETHOD(InsertParagraphAfter)();
			STDMETHOD(ConvertToTableOld)(VARIANT* Separator, VARIANT* NumRows, VARIANT* NumColumns, VARIANT* InitialColumnWidth, VARIANT* Format, VARIANT* ApplyBorders, VARIANT* ApplyShading, VARIANT* ApplyFont, VARIANT* ApplyColor, VARIANT* ApplyHeadingRows, VARIANT* ApplyLastRow, VARIANT* ApplyFirstColumn, VARIANT* ApplyLastColumn, VARIANT* AutoFit, IDispatch** prop);
			STDMETHOD(InsertDateTimeOld)(VARIANT* DateTimeFormat, VARIANT* InsertAsField, VARIANT* InsertAsFullWidth);
			STDMETHOD(InsertSymbol)(long CharacterNumber, VARIANT* Font, VARIANT* Unicode, VARIANT* Bias);
			STDMETHOD(InsertCrossReference_2002)(VARIANT* ReferenceType, WdReferenceKind ReferenceKind, VARIANT* ReferenceItem, VARIANT* InsertAsHyperlink, VARIANT* IncludePosition);
			STDMETHOD(InsertCaptionXP)(VARIANT* Label, VARIANT* Title, VARIANT* TitleAutoText, VARIANT* Position);
			STDMETHOD(CopyAsPicture)();
			STDMETHOD(SortOld)(VARIANT* ExcludeHeader, VARIANT* FieldNumber, VARIANT* SortFieldType, VARIANT* SortOrder, VARIANT* FieldNumber2, VARIANT* SortFieldType2, VARIANT* SortOrder2, VARIANT* FieldNumber3, VARIANT* SortFieldType3, VARIANT* SortOrder3, VARIANT* SortColumn, VARIANT* Separator, VARIANT* CaseSensitive, VARIANT* LanguageID);
			STDMETHOD(SortAscending)();
			STDMETHOD(SortDescending)();
			STDMETHOD(IsEqual)(Range* Range, VARIANT_BOOL* prop);
			STDMETHOD(Calculate)(float* prop);
			STDMETHOD(GoTo)(VARIANT* What, VARIANT* Which, VARIANT* Count, VARIANT* Name, Range** prop);
			STDMETHOD(GoToNext)(WdGoToItem What, Range** prop);
			STDMETHOD(GoToPrevious)(WdGoToItem What, Range** prop);
			STDMETHOD(PasteSpecial)(VARIANT* IconIndex, VARIANT* Link, VARIANT* Placement, VARIANT* DisplayAsIcon, VARIANT* DataType, VARIANT* IconFileName, VARIANT* IconLabel);
			STDMETHOD(LookupNameProperties)();
			STDMETHOD(ComputeStatistics)(WdStatistic Statistic, long* prop);
			STDMETHOD(Relocate)(long Direction);
			STDMETHOD(CheckSynonyms)();
			STDMETHOD(SubscribeTo)(BSTR Edition, VARIANT* Format);
			STDMETHOD(CreatePublisher)(VARIANT* Edition, VARIANT* ContainsPICT, VARIANT* ContainsRTF, VARIANT* ContainsText);
			STDMETHOD(InsertAutoText)();
			STDMETHOD(InsertDatabase)(VARIANT* Format, VARIANT* Style, VARIANT* LinkToSource, VARIANT* Connection, VARIANT* SQLStatement, VARIANT* SQLStatement1, VARIANT* PasswordDocument, VARIANT* PasswordTemplate, VARIANT* WritePasswordDocument, VARIANT* WritePasswordTemplate, VARIANT* DataSource, VARIANT* From, VARIANT* To, VARIANT* IncludeFields);
			STDMETHOD(AutoFormat)();
			STDMETHOD(CheckGrammar)();
			STDMETHOD(CheckSpelling)(VARIANT* CustomDictionary, VARIANT* IgnoreUppercase, VARIANT* AlwaysSuggest, VARIANT* CustomDictionary2, VARIANT* CustomDictionary3, VARIANT* CustomDictionary4, VARIANT* CustomDictionary5, VARIANT* CustomDictionary6, VARIANT* CustomDictionary7, VARIANT* CustomDictionary8, VARIANT* CustomDictionary9, VARIANT* CustomDictionary10);
			STDMETHOD(GetSpellingSuggestions)(VARIANT* CustomDictionary, VARIANT* IgnoreUppercase, VARIANT* MainDictionary, VARIANT* SuggestionMode, VARIANT* CustomDictionary2, VARIANT* CustomDictionary3, VARIANT* CustomDictionary4, VARIANT* CustomDictionary5, VARIANT* CustomDictionary6, VARIANT* CustomDictionary7, VARIANT* CustomDictionary8, VARIANT* CustomDictionary9, VARIANT* CustomDictionary10, IDispatch** prop);
			STDMETHOD(InsertParagraphBefore)();
			STDMETHOD(NextSubdocument)();
			STDMETHOD(PreviousSubdocument)();
			STDMETHOD(ConvertHangulAndHanja)(VARIANT* ConversionsMode, VARIANT* FastConversion, VARIANT* CheckHangulEnding, VARIANT* EnableRecentOrdering, VARIANT* CustomDictionary);
			STDMETHOD(PasteAsNestedTable)();
			STDMETHOD(ModifyEnclosure)(VARIANT* Style, VARIANT* Symbol, VARIANT* EnclosedText);
			STDMETHOD(PhoneticGuide)(BSTR Text, WdPhoneticGuideAlignmentType Alignment, long Raise, long FontSize, BSTR FontName);
			STDMETHOD(InsertDateTime)(VARIANT* DateTimeFormat, VARIANT* InsertAsField, VARIANT* InsertAsFullWidth, VARIANT* DateLanguage, VARIANT* CalendarType);
			STDMETHOD(Sort)(VARIANT* ExcludeHeader, VARIANT* FieldNumber, VARIANT* SortFieldType, VARIANT* SortOrder, VARIANT* FieldNumber2, VARIANT* SortFieldType2, VARIANT* SortOrder2, VARIANT* FieldNumber3, VARIANT* SortFieldType3, VARIANT* SortOrder3, VARIANT* SortColumn, VARIANT* Separator, VARIANT* CaseSensitive, VARIANT* BidiSort, VARIANT* IgnoreThe, VARIANT* IgnoreKashida, VARIANT* IgnoreDiacritics, VARIANT* IgnoreHe, VARIANT* LanguageID);
			STDMETHOD(DetectLanguage)();
			STDMETHOD(ConvertToTable)(VARIANT* Separator, VARIANT* NumRows, VARIANT* NumColumns, VARIANT* InitialColumnWidth, VARIANT* Format, VARIANT* ApplyBorders, VARIANT* ApplyShading, VARIANT* ApplyFont, VARIANT* ApplyColor, VARIANT* ApplyHeadingRows, VARIANT* ApplyLastRow, VARIANT* ApplyFirstColumn, VARIANT* ApplyLastColumn, VARIANT* AutoFit, VARIANT* AutoFitBehavior, VARIANT* DefaultTableBehavior, IDispatch** prop);
			STDMETHOD(TCSCConverter)(WdTCSCConverterDirection WdTCSCConverterDirection, VARIANT_BOOL CommonTerms, VARIANT_BOOL UseVariants);
			STDMETHOD(get_LanguageDetected)(VARIANT_BOOL* prop);
			STDMETHOD(put_LanguageDetected)(VARIANT_BOOL prop);
			STDMETHOD(get_FitTextWidth)(float* prop);
			STDMETHOD(put_FitTextWidth)(float prop);
			STDMETHOD(get_HorizontalInVertical)(WdHorizontalInVerticalType* prop);
			STDMETHOD(put_HorizontalInVertical)(WdHorizontalInVerticalType prop);
			STDMETHOD(get_TwoLinesInOne)(WdTwoLinesInOneType* prop);
			STDMETHOD(put_TwoLinesInOne)(WdTwoLinesInOneType prop);
			STDMETHOD(get_CombineCharacters)(VARIANT_BOOL* prop);
			STDMETHOD(put_CombineCharacters)(VARIANT_BOOL prop);
			STDMETHOD(get_NoProofing)(long* prop);
			STDMETHOD(put_NoProofing)(long prop);
			STDMETHOD(get_TopLevelTables)(IDispatch** prop);
			STDMETHOD(get_Scripts)(IDispatch** prop);
			STDMETHOD(get_CharacterWidth)(WdCharacterWidth* prop);
			STDMETHOD(put_CharacterWidth)(WdCharacterWidth prop);
			STDMETHOD(get_Kana)(WdKana* prop);
			STDMETHOD(put_Kana)(WdKana prop);
			STDMETHOD(get_BoldBi)(long* prop);
			STDMETHOD(put_BoldBi)(long prop);
			STDMETHOD(get_ItalicBi)(long* prop);
			STDMETHOD(put_ItalicBi)(long prop);
			STDMETHOD(get_ID)(BSTR* prop);
			STDMETHOD(put_ID)(BSTR prop);
			STDMETHOD(get_HTMLDivisions)(IDispatch** prop);
			STDMETHOD(get_SmartTags)(IDispatch** prop);
			STDMETHOD(get_ShowAll)(VARIANT_BOOL* prop);
			STDMETHOD(put_ShowAll)(VARIANT_BOOL prop);
			STDMETHOD(get_Document)(_Document** prop);
			STDMETHOD(get_FootnoteOptions)(IDispatch** prop);
			STDMETHOD(get_EndnoteOptions)(IDispatch** prop);
            STDMETHOD(PasteAndFormat)(WdRecoveryType Type);
            STDMETHOD(PasteExcelTable)(VARIANT_BOOL LinkedToExcel, VARIANT_BOOL WordFormatting, VARIANT_BOOL RTF);
			STDMETHOD(PasteAppendTable)();
			STDMETHOD(get_XMLNodes)(IDispatch** prop);
			STDMETHOD(get_XMLParentNode)(IDispatch** prop);
			STDMETHOD(get_Editors)(IDispatch** prop);
			STDMETHOD(get_XML)(VARIANT_BOOL DataOnly, BSTR* prop);
			STDMETHOD(get_EnhMetaFileBits)(VARIANT* prop);
			STDMETHOD(GoToEditableRange)(VARIANT* EditorID, Range** prop);
			STDMETHOD(InsertXML)(BSTR XML, VARIANT* Transform);
			STDMETHOD(InsertCaption)(VARIANT* Label, VARIANT* Title, VARIANT* TitleAutoText, VARIANT* Position, VARIANT* ExcludeLabel);
			STDMETHOD(InsertCrossReference)(VARIANT* ReferenceType, WdReferenceKind ReferenceKind, VARIANT* ReferenceItem, VARIANT* InsertAsHyperlink, VARIANT* IncludePosition, VARIANT* SeparateNumbers, VARIANT* SeparatorString);

			void SetWordDocument(CWordDocument*	pDocument);
			void SetStyle(CWordStyle* pStyle);
			CWordStyle* GetStyle() const;

		private:
			CComPtr<CWordStyle>	m_pStyle;
			CWordDocument* m_pDocument;
            long m_Start;
            long m_End;
		};

		typedef CComObject<CWordRange> WordRangeObj;
		typedef CComPtr<CWordRange> WordRangePtr;
	}
}

#endif //_WORD_RANGE_H_