
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/


#pragma once

#include "OODocument.h"

#include <string>
#include <vector>

/*
	COOAplication is a bridge to the Open Office. It provides a functionality similar to Word Application class
	by utilizing OpenOffice API.
*/

class COOApplication
{
	IDispatch  *m_pdOOApp;	// (IDispatch) OpenOffice application
	IDispatch  *m_pdDesktop;// (IDispatch) OpenOffice desktop

    IDispatch  *m_pdDatabaseContext;
    IDispatch  *m_pdActiveConnection;
public:
	COOApplication(void);
	~COOApplication(void);

	HRESULT CreateApplication	();
	HRESULT DestroyApplication	();

	HRESULT Attach	(IDispatch  *m_pdOOApp, IDispatch  *m_pdDesktop);
	HRESULT Attach	(COOApplication *pApp);
	HRESULT Detach	();

	HRESULT GetDispatchPropertyValue	(IDispatch** pIDispatch);
	HRESULT SetPropertyValue			(IDispatch *pdProperty, const std::wstring &sName, const CComVariant &rValue);

	/*
		Create new document
	*/
	HRESULT CreateBlankDocument			(const std::wstring &sDocumentType, COODocument **pDocument, BOOL bShowed);
	/*
		Open existing document
	*/
	HRESULT FileOpen					(CComVariant vName, COODocument **pDocument, BOOL bShowed);

	// Work with OO DataSource
    HRESULT CreateDatabaseContext       ();
    HRESULT CreateTextDataSource        (CComVariant vDSName,
                                         CComVariant vPathName);
    HRESULT RemoveDataSource            (CComVariant vDSName);
    HRESULT IsExist                     (CComVariant vDSName,
                                         BOOL &bResult);
    static HRESULT SetURL               (IDispatch *pDatasource, CComVariant vUrl);
	/*
		Convert Windows path to database URL 
	*/
    static void    ConvertPathToDSUrl   (const std::wstring &sPath, std::wstring &sURL);
	/*
		Convert Windows path to URL 
	*/
    static void    ConvertPathToUrl     (const std::wstring &sPath, std::wstring &sURL);
	/*
		OO MailMerge
	*/
    HRESULT MailMerge                   (CComVariant vDSName,
                                         CComVariant vTemplateName,
                                         CComVariant vDestinationName,
                                         CComVariant vTableName,
                                         CComVariant vResultFileName);

    HRESULT SetMailMergeProperty        (IDispatch   *pdMergeObject,
                                         CComVariant vName,
                                         CComVariant vValue);
    HRESULT CreateMailMergeService      (IDispatch **ppResult);

	/* 
		Combine set of documents
	*/
    HRESULT CombineDocuments            (const std::vector<std::wstring> &vFileList, const std::wstring &vDestinationFileName, BOOL bShowed);

};
