
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/



#include "OOApplication.h"
#include "OODocument.h"

#include <AutoWrapper.h>

const std::wstring COODocument::TypeCalcDocument	= L"private:factory/scalc";
const std::wstring COODocument::TypeWriterDocument	= L"private:factory/swriter";

using namespace AutomateIT::Utils;


COODocument::COODocument(COOApplication *pParentApplication, IDispatch *pdDoc)
{
	if (pdDoc != NULL)
	{
		m_pdDoc = pdDoc;
		m_pdDoc->AddRef();
	} else 
		m_pdDoc = NULL;

	m_pApp = new COOApplication();
	if (m_pApp != NULL)
    {
		m_pApp->Attach(pParentApplication);
    }

}
HRESULT COODocument::SaveFile(CComVariant vFileName, CComVariant vFormatName)

{
    VARIANT vRes;
	IDispatch *pdProperty = NULL;
    CComVariant vParam;
    HRESULT hres;
    SAFEARRAY FAR* pPropVals;

    if (vFormatName.vt != VT_ERROR)
    {
        hres = m_pApp->GetDispatchPropertyValue(&pdProperty); 
        if (hres != S_OK)
        {
            return hres;
        }
	    // Set PropertyValue data
        hres = m_pApp->SetPropertyValue(pdProperty, L"FilterName", vFormatName);
        if (hres != S_OK)
        {
            pdProperty->Release();
            return hres;
        }

	    // Init params
	    pPropVals = SafeArrayCreateVector( VT_DISPATCH, 0, 1 );
	    long ix = 0; 
	    HRESULT hres = SafeArrayPutElement( pPropVals, &ix, pdProperty); 
	    
	    vParam.vt = VT_DISPATCH | VT_ARRAY;
	    vParam.parray = pPropVals;
    } else 
    {
        vParam.vt = VT_ARRAY | VT_VARIANT;
        vParam.parray = NULL;
    }
	// Call StoreToURL for save document to file
	hres = AutoWrapper::Execute(DISPATCH_METHOD, &vRes, m_pdDoc, L"storeToURL", 2, vParam, vFileName);
    if (pdProperty != NULL)
    {
        pdProperty->Release();
        SafeArrayDestroy(pPropVals);
    }
    return hres;
}

HRESULT COODocument::Close()
{
//  Code for close
//	AutoWrapper::Execute(DISPATCH_METHOD, &res, this->m_pdDoc, L"Dispose", 0);
//	dpv->Release ();
    if (m_pdDoc == NULL)
    {
        return E_INVALIDARG;
    }
    VARIANT vRes;
    return AutoWrapper::Execute(DISPATCH_METHOD, &vRes, m_pdDoc, L"dispose", 0);
}

COODocument::~COODocument(void)
{
	if (m_pApp != NULL)
	{
		m_pApp->Detach ();
	}
}
